@@ -175,7 +175,8 @@ headers and provide form."
     (package-lint-test--run "" :url ";; URL: git://test/test.git\n"))))
 
 (ert-deftest package-lint-test-accept-homepage ()
-  (should (equal '() (package-lint-test--run "" :url ";; Homepage: https://package-lint.test/foo\n"))))
+  (should (equal '() (package-lint-test--run "" :url ";; Homepage: https://package-lint.test/foo\n")))
+  (should (equal '() (package-lint-test--run "" :url ";; Homepage: <https://package-lint.test/foo>\n"))))
 
 (ert-deftest package-lint-test-warn-invalid-version ()
   (should
@@ -253,10 +254,6 @@ headers and provide form."
   (should (equal '() (package-lint-test--run "" :header ";;; test.el --- Something for .emacs\n"))))
 
 (ert-deftest package-lint-test-error-invalid-dependency ()
-  (should
-   (member
-    '(6 0 error "Expected (package-name \"version-num\"), but found invalid.")
-    (package-lint-test--run ";; Package-Requires: (invalid)")))
   (should
    (member
     '(6 23 error "\"invalid\" is not a valid version string: see `version-to-list'.")
@@ -268,11 +265,11 @@ headers and provide form."
     '((6 23 error "You can only depend on Emacs version 24 or greater: package.el for Emacs 23 does not support the \"emacs\" pseudopackage."))
     (package-lint-test--run ";; Package-Requires: ((emacs \"23\"))"))))
 
-(ert-deftest package-lint-test-warning-emacs-28-dep ()
+(ert-deftest package-lint-test-warning-emacs-head-dep ()
   (should
    (equal
     '((6 23 warning "This makes the package uninstallable in all released Emacs versions."))
-    (package-lint-test--run ";; Package-Requires: ((emacs \"28\"))"))))
+    (package-lint-test--run ";; Package-Requires: ((emacs \"30\"))"))))
 
 (ert-deftest package-lint-test-accept-emacs-24+-dep ()
   (should (equal '() (package-lint-test--run ";; Package-Requires: ((emacs \"24\"))")))
@@ -291,6 +288,14 @@ headers and provide form."
     (package-lint-test--run ";; Package-Requires: ((package-lint \"20160101.1234\"))"))))
 
 (ert-deftest package-lint-test-warn-unversioned-dep ()
+  (should
+   (member
+    '(6 22 warning "Use a properly versioned dependency on \"package-lint\" if possible.")
+    (package-lint-test--run ";; Package-Requires: (package-lint)")))
+  (should
+   (member
+    '(6 23 warning "Use a properly versioned dependency on \"package-lint\" if possible.")
+    (package-lint-test--run ";; Package-Requires: ((package-lint))")))
   (should
    (equal
     '((6 23 warning "Use a properly versioned dependency on \"package-lint\" if possible."))
@@ -420,6 +425,12 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     (when-let blah)
   (bloop))"))))
 
+(ert-deftest package-lint-test-distinguishes-let-vars-from-function-calls ()
+  (should (equal '() (package-lint-test--run "(let (seq-length '(foo)))")))
+  (should (equal '() (package-lint-test--run "(let ((seq-length '(foo))))")))
+  (should (equal '() (package-lint-test--run "(let* ((seq-length '(foo))))")))
+  (should (equal '() (package-lint-test--run "(let* (bar (seq-length '(foo))))"))))
+
 (ert-deftest package-lint-test-error-new-backported-functions ()
   (should
    (equal
@@ -485,6 +496,9 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
                       "(defun test-foo ())\n(defun test ())")))
   (should (equal '() (package-lint-test--run
                       "(define-globalized-minor-mode global-test-mode ignore ignore :require 'test)")))
+  (should (equal '() (package-lint-test--run
+                      "(define-globalized-minor-mode global-test-mode ignore ignore :require 'test-mode)"
+                      :featurename 'test-mode)))
   (should (equal '() (package-lint-test--run
                       "(define-globalized-minor-mode global-test-foo-mode ignore ignore :require 'test)"))))
 
@@ -719,6 +733,20 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     '()
     (package-lint-test--run "(defvaralias 'test-foobar 'string-equal)"))))
 
+(ert-deftest package-lint-test-no-error-for-obsolete-alias-name ()
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(define-obsolete-function-alias 'foobar 'string-equal)")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(define-obsolete-variable-alias 'foobar 'string-equal)")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(define-obsolete-face-alias 'foobar 'string-equal)"))))
+
 (ert-deftest package-lint-test-error-format-string ()
   (should
    (equal
@@ -802,5 +830,33 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     '((1 0 error "There is no (provide-theme 'foo) form."))
     (package-lint-test--run "(provide-theme 'bar)" :provide "" :featurename "foo-theme"))))
 
+(defmacro package-lint-test--bufsim (contents &rest body)
+  `(with-temp-buffer
+     (insert ,contents)
+     (set (make-local-variable 'delay-mode-hooks) t)
+     (emacs-lisp-mode)
+     (search-backward-regexp "|")
+     (delete-char 1)
+     ,@body))
+
+(ert-deftest package-lint-test--let-binding-detection ()
+  (should
+   (package-lint-test--bufsim
+    "(let (|foo) blah)"
+    (package-lint--is-a-let-binding)))
+  (should
+   (package-lint-test--bufsim
+    "(let (bar |foo) blah)"
+    (package-lint--is-a-let-binding)))
+  (should
+   (package-lint-test--bufsim
+    "(let ((|foo bar)) blah)"
+    (package-lint--is-a-let-binding)))
+  (should-not
+   (package-lint-test--bufsim
+    "(let ((bar |foo)) blah)"
+    (package-lint--is-a-let-binding))))
+
+
 (provide 'package-lint-test)
 ;;; package-lint-test.el ends here
