@@ -6,7 +6,7 @@
 ;;         Fanael Linithien <fanael4@gmail.com>
 ;; URL: https://github.com/purcell/package-lint
 ;; Keywords: lisp
-;; Version: 0
+;; Version: 0.19
 ;; Package-Requires: ((cl-lib "0.5") (emacs "24.1") (let-alist "1.0.6"))
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -86,6 +86,9 @@ The path can be absolute or relative to that of the linted file.")
   "List of errors and warnings for the current buffer.
 This is bound dynamically while the checks run.")
 
+(defconst package-lint-emacs-head-version '(30)
+  "Version of Emacs HEAD.")
+
 (defconst package-lint-backport-libraries
   (list (list 'cl-lib "\\`cl-")
         (list 'cl-generic "\\`cl-\\(?:def\\)?generic")
@@ -95,6 +98,8 @@ This is bound dynamically while the checks run.")
         (list 'nadvice "\\`advice-")
         (list 'ol "\\`org-link-" 'org "9.3")
         (list 'seq "\\`seq-")
+        (list 'transient "\\`transient-")
+        (list 'hierarchy "\\`hierarchy-")
         (list 'let-alist "\\`let-alist"))
   "A sequence of (FEATURE SYMBOL-NAME-MATCH) for backport libraries.
 These are libraries that are built into newer Emacsen and also
@@ -128,7 +133,7 @@ optional minimum version containing the feature.")
     info)
   "A hash table from SYMBOL to a list of events in its history.
 Each event is of the form (ACTION . EMACS-VER), where ACTION is a
-symbol such as 'variable-added.")
+symbol such as `variable-added'.")
 
 (defun package-lint-symbol-info (sym)
   "Retrieve information about SYM, as an alist of (ACTION . EMACS-VER)."
@@ -163,7 +168,7 @@ symbol such as 'variable-added.")
                       "\n"))))
 
 (defun package-lint--added-or-removed-function-p (sym)
-  "Predicate that returns t if SYM is a function added/removed in any known emacs version."
+  "Return t if SYM is a function added/removed in any known Emacs version."
   (let-alist (package-lint-symbol-info sym)
     (or .function-added .function-removed)))
 
@@ -176,11 +181,15 @@ symbol such as 'variable-added.")
     "org-babel-prep-session:"
     "org-babel-variable-assignments:"
     "org-babel-default-header-args:"
-    "pcomplete/"))
+    "pcomplete/"
+    "use-package-normalize/"
+    "use-package-handler/"
+    "use-package-autoloads/"))
   "A regexp matching whitelisted non-standard symbol prefixes.")
 
 (defvar package-lint--allowed-prefix-mappings
   '(("ob-" . ("org-"))
+    ("oc-" . ("org-"))
     ("ol-" . ("org-"))
     ("ox-" . ("org-")))
   "Alist containing mappings of package prefixes to symbol prefixes.")
@@ -298,11 +307,11 @@ POS defaults to `point'."
     (package-lint--error (line-number-at-pos) (- (point) (line-beginning-position)) type message)))
 
 (defun package-lint--error-at-bol (type message)
-  "Construct a datum for error at the beginning of the current line with TYPE and MESSAGE."
+  "Construct an error at the beginning of the current line with TYPE and MESSAGE."
   (package-lint--error (line-number-at-pos) 0 type message))
 
 (defun package-lint--error-at-bob (type message)
-  "Construct a datum for error at the beginning of the buffer with TYPE and MESSAGE."
+  "Construct an error at the beginning of the buffer with TYPE and MESSAGE."
   (package-lint--error 1 0 type message))
 
 
@@ -380,6 +389,10 @@ Instead it should use `user-emacs-directory' or `locate-user-emacs-file'."
   (if (package-lint--goto-header "\\(?:URL\\|Homepage\\)")
       (let ((url (match-string 3))
             (url-start (match-beginning 3)))
+        (when (string-match-p "^<.*>$" url)
+          (setq url (substring url 1 -1)
+                url-start (1+ url-start))
+          (backward-char 1))
         (unless (and (equal (thing-at-point 'url) url)
                      (string-match-p "^https?://" url))
           (package-lint--error-at-point
@@ -424,20 +437,30 @@ the form (PACKAGE-NAME PACKAGE-VERSION DEP-POSITION)."
   (let (valid-deps)
     (dolist (entry parsed-deps)
       (pcase entry
-        ((and `(,package-name ,package-version)
-              (guard (symbolp package-name))
-              (guard (stringp package-version)))
+        ((or (and `(,package-name ,package-version)
+                  (guard (symbolp package-name))
+                  (guard (stringp package-version)))
+             (and `(,package-name)
+                  (guard (symbolp package-name)))
+             (and package-name
+                  (guard (symbolp package-name))))
          ;; Find the column at which the dependency is declared so we can
          ;; properly report the position of errors.
          (let ((dep-pos
                 (save-excursion
                   (goto-char position)
-                  (let ((pattern
-                         (format "( *\\(%s\\)\\(?:)\\|[^[:alnum:]_\\-].*?)\\)"
-                                 (regexp-quote (symbol-name package-name)))))
-                    (if (re-search-forward pattern (line-end-position) t)
+                  (let* ((symbol-pattern
+                          (format "\\(%s\\)"
+                                  (regexp-quote (symbol-name package-name))))
+                         (list-pattern
+                          (format "( *%s\\(?:[^[:alnum:]_\\-].*?\\)?)"
+                                  symbol-pattern)))
+                    (if (or (re-search-forward list-pattern (line-end-position) t)
+                            (re-search-forward symbol-pattern (line-end-position) t))
                         (match-beginning 1)
                       position)))))
+           (unless package-version
+             (setq package-version "0"))
            (if (ignore-errors (version-to-list package-version))
                (push (list package-name
                            (version-to-list package-version)
@@ -478,7 +501,7 @@ required version PACKAGE-VERSION.  If not, raise an error for DEP-POS."
          'error
          "You can only depend on Emacs version 24 or greater: package.el for Emacs 23 does not support the \"emacs\" pseudopackage."
          dep-pos))
-       ((version-list-<= '(28) package-version)
+       ((version-list-<= package-lint-emacs-head-version package-version)
         (package-lint--error-at-point
          'warning
          "This makes the package uninstallable in all released Emacs versions."
@@ -562,6 +585,26 @@ CALLBACK."
               (unless (package-lint--inside-comment-or-string-p)
                 (apply #'package-lint--error-at-point err)))))))))
 
+(defun package-lint--quick-up-list ()
+  (if (eq ?\( (char-before))
+      (backward-char 1)
+    (backward-up-list)))
+
+(defun package-lint--is-a-let-binding ()
+  "Return non-nil if point is on a symbol being introduced by a let binding."
+  (let ((binding-pat (rx "(" (or "let" "let*" "cl-let") symbol-end)))
+    (condition-case _
+        (save-excursion
+          (save-match-data
+            (let ((was-first (eq ?\( (char-before))))
+              (package-lint--quick-up-list)
+              (package-lint--quick-up-list)
+              (or (looking-at binding-pat)
+                  (when was-first
+                    (package-lint--quick-up-list)
+                    (looking-at binding-pat))))))
+      (scan-error nil))))
+
 (defun package-lint--check-version-regexp-list (valid-deps symbol-regexp type)
   "Warn if symbols matched by SYMBOL-REGEXP are unavailable in the target Emacs.
 The target Emacs version is taken from VALID-DEPS, which are the
@@ -572,46 +615,48 @@ type of the symbol, either FUNCTION or FEATURE."
      symbol-regexp
      (lambda (sym)
        (let-alist (package-lint-symbol-info sym)
-         (let ((added-in-version (cl-ecase type
-                                   ('function .function-added)
-                                   ('feature .library-added))))
+         (let ((added-in-version (pcase type
+                                   (`function .function-added)
+                                   (`feature .library-added))))
            (when (and added-in-version (version-list-< emacs-version-dep added-in-version))
-             (unless (and (eq type 'function) (package-lint--seen-fboundp-check-for sym))
-               (let* ((available-backport-with-ver
-                       (cl-ecase type
-                         ('feature
-                          (cl-some (lambda (bp)
-                                     (when (string= (car bp) sym)
-                                       (or (cddr bp)
-                                           (list (car bp)))))
-                                   package-lint-backport-libraries))
-                         ('function
-                          (cl-some (lambda (bp)
-                                     (when (string-match-p (nth 1 bp) sym)
-                                       (or (cddr bp)
-                                           (list (car bp)))))
-                                   package-lint-backport-libraries))))
-                      (available-backport (car available-backport-with-ver))
-                      (required-backport-version (cadr available-backport-with-ver))
-                      (matching-dep (when available-backport
-                                      (assoc available-backport valid-deps))))
-                 (unless (and matching-dep
-                              (or (not required-backport-version)
-                                  (version-list-<= (version-to-list required-backport-version)
-                                                   (cadr matching-dep))))
-                   (list
-                    'error
-                    (format "You should depend on (emacs \"%s\")%s if you need `%s'."
-                            (mapconcat #'number-to-string added-in-version ".")
-                            (if available-backport
-                                (format " or the %s package"
-                                        (if required-backport-version
-                                            (format "(%s \"%s\")"
-                                                    available-backport
-                                                    required-backport-version)
-                                          available-backport))
-                              "")
-                            sym))))))))))))
+             (let* ((available-backport-with-ver
+                     (pcase type
+                       (`feature
+                        (cl-some (lambda (bp)
+                                   (when (string= (car bp) sym)
+                                     (or (cddr bp)
+                                         (list (car bp)))))
+                                 package-lint-backport-libraries))
+                       (`function
+                        (cl-some (lambda (bp)
+                                   (when (string-match-p (nth 1 bp) sym)
+                                     (or (cddr bp)
+                                         (list (car bp)))))
+                                 package-lint-backport-libraries))))
+                    (available-backport (car available-backport-with-ver))
+                    (required-backport-version (cadr available-backport-with-ver))
+                    (matching-dep (when available-backport
+                                    (assoc available-backport valid-deps))))
+               (unless (or (and matching-dep
+                                (or (not required-backport-version)
+                                    (version-list-<= (version-to-list required-backport-version)
+                                                     (cadr matching-dep))))
+                           (and (eq type 'function)
+                                (or (package-lint--seen-fboundp-check-for sym)
+                                    (package-lint--is-a-let-binding))))
+                 (list
+                  'error
+                  (format "You should depend on (emacs \"%s\")%s if you need `%s'."
+                          (mapconcat #'number-to-string added-in-version ".")
+                          (if available-backport
+                              (format " or the %s package"
+                                      (if required-backport-version
+                                          (format "(%s \"%s\")"
+                                                  available-backport
+                                                  required-backport-version)
+                                        available-backport))
+                            "")
+                          sym)))))))))))
 
 (defun package-lint--check-eval-after-load ()
   "Warn about use of `eval-after-load' and co."
@@ -648,7 +693,8 @@ type of the symbol, either FUNCTION or FEATURE."
   "Regexp to match unconditional `require' forms.")
 
 (defun package-lint--check-libraries-available-in-emacs (valid-deps)
-  "Warn about use of libraries that are not available in the Emacs version in VALID-DEPS."
+  "Warn about use of libraries that are not available.
+The check is performed against the Emacs version in VALID-DEPS."
   (package-lint--check-version-regexp-list
    valid-deps
    package-lint--unconditional-require-regexp
@@ -683,7 +729,8 @@ type of the symbol, either FUNCTION or FEATURE."
   "Regexp to match function names.")
 
 (defun package-lint--check-macros-functions-available-in-emacs (valid-deps)
-  "Warn about use of functions/macros that are not available in the Emacs version in VALID-DEPS."
+  "Warn about use of unavailable functions/macros that are not available.
+The check is performed against the Emacs version in VALID-DEPS."
   (package-lint--check-version-regexp-list
    valid-deps
    package-lint--function-name-regexp
@@ -750,7 +797,8 @@ the Emacs dependency matches the re-addition."
              "`lexical-binding' must be set in the first line.")))))))
 
 (defun package-lint--check-cl-lib-version (valid-deps)
-  "Check that any dependency in VALID-DEPS on \"cl-lib\" is on a remotely-installable version."
+  "Check dependencies in VALID-DEPS on \"cl-lib\".
+The specified versions must be remotely available for installation."
   (let ((emacs-version-dep (or (nth 1 (assq 'emacs valid-deps)) '(0)))
         (cl-lib-dep (assq 'cl-lib valid-deps)))
     (when cl-lib-dep
@@ -790,7 +838,14 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
       (insert-buffer-substring-no-properties orig-buffer)
       (goto-char (point-min))
       (package-lint--update-or-insert-version "0")
-      (package-buffer-info))))
+      (let ((info (package-buffer-info)))
+        ;; Pedantically check for the footer comment
+        ;; In Emacs 29 this became a soft failure, but for packages to
+        ;; be installable with the package.el in older Emacsen, the
+        ;; footer comment must be there nonetheless.
+        (goto-char (point-min))
+        (search-forward (format ";;; %s.el ends here" (package-lint--package-desc-name info)))
+        info))))
 
 (defun package-lint--check-package-el-can-parse ()
   "Check that `package-buffer-info' can read metadata from this file.
@@ -915,7 +970,7 @@ Valid definition names are:
       (when position
         (goto-char position)
         (looking-at-p (rx (*? space) "(" (*? space)
-                          (or "defadvice" "cl-defmethod" "define-advice")
+                          (or "defadvice" "cl-defmethod" "define-advice" "defalias" "defvaralias" (seq "define-obsolete-" (or "face" "function" "variable") "-alias"))
                           symbol-end)))))
 
 (defun package-lint--check-defs-prefix (prefix definitions)
@@ -1196,7 +1251,7 @@ Returns a list, each element of which is list of
 
    (LINE COL TYPE MESSAGE)
 
-where TYPE is either 'warning or 'error.
+where TYPE is either `warning' or `error'.
 
 Current buffer is used if none is specified."
   (with-current-buffer (or buffer (current-buffer))
@@ -1224,7 +1279,7 @@ Current buffer is used if none is specified."
     (display-buffer buf)))
 
 (defgroup package-lint nil
-  "A linting library for elisp package authors"
+  "A linting library for elisp package authors."
   :group 'development)
 
 (defcustom package-lint-batch-fail-on-warnings t
